'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _util = require('./util');

var _pipeline = require('./pipeline');

/**
 * A RouteTransition object manages the pipeline of a
 * router-view switching process. This is also the object
 * passed into user route hooks.
 *
 * @param {Router} router
 * @param {Route} to
 * @param {Route} from
 */

var RouteTransition = (function () {
  function RouteTransition(router, to, from) {
    _classCallCheck(this, RouteTransition);

    this.router = router;
    this.to = to;
    this.from = from;
    this.next = null;
    this.aborted = false;

    // start by determine the queues

    // the deactivate queue is an array of router-view
    // directive instances that need to be deactivated,
    // deepest first.
    this.deactivateQueue = router._views;

    // check the default handler of the deepest match
    var matched = to._matched ? Array.prototype.slice.call(to._matched) : [];

    // the activate queue is an array of route handlers
    // that need to be activated
    this.activateQueue = matched.map(function (match) {
      return match.handler;
    });
  }

  /**
   * Abort current transition and return to previous location.
   */

  _createClass(RouteTransition, [{
    key: 'abort',
    value: function abort() {
      if (!this.aborted) {
        this.aborted = true;
        // if the root path throws an error during validation
        // on initial load, it gets caught in an infinite loop.
        var abortingOnLoad = !this.from.path && this.to.path === '/';
        if (!abortingOnLoad) {
          this.router.replace(this.from.path || '/');
        }
      }
    }

    /**
     * Abort current transition and redirect to a new location.
     *
     * @param {String} path
     */

  }, {
    key: 'redirect',
    value: function redirect(path) {
      if (!this.aborted) {
        this.aborted = true;
        path = (0, _util.mapParams)(path, this.to.params, this.to.query);
        this.router.replace(path);
      }
    }

    /**
     * A router view transition's pipeline can be described as
     * follows, assuming we are transitioning from an existing
     * <router-view> chain [Component A, Component B] to a new
     * chain [Component A, Component C]:
     *
     *  A    A
     *  | => |
     *  B    C
     *
     * 1. Reusablity phase:
     *   -> canReuse(A, A)
     *   -> canReuse(B, C)
     *   -> determine new queues:
     *      - deactivation: [B]
     *      - activation: [C]
     *
     * 2. Validation phase:
     *   -> canDeactivate(B)
     *   -> canActivate(C)
     *
     * 3. Activation phase:
     *   -> deactivate(B)
     *   -> activate(C)
     *
     * Each of these steps can be asynchronous, and any
     * step can potentially abort the transition.
     *
     * @param {Function} cb
     */

  }, {
    key: 'start',
    value: function start(cb) {
      var transition = this;
      var daq = this.deactivateQueue;
      var aq = this.activateQueue;
      var rdaq = daq.slice().reverse();
      var reuseQueue = undefined;

      // 1. Reusability phase
      var i = undefined;
      for (i = 0; i < rdaq.length; i++) {
        if (!(0, _pipeline.canReuse)(rdaq[i], aq[i], transition)) {
          break;
        }
      }
      if (i > 0) {
        reuseQueue = rdaq.slice(0, i);
        daq = rdaq.slice(i).reverse();
        aq = aq.slice(i);
      }

      // 2. Validation phase
      transition.runQueue(daq, _pipeline.canDeactivate, function () {
        transition.runQueue(aq, _pipeline.canActivate, function () {
          transition.runQueue(daq, _pipeline.deactivate, function () {
            // 3. Activation phase

            // Update router current route
            transition.router._onTransitionValidated(transition);

            // trigger reuse for all reused views
            reuseQueue && reuseQueue.forEach(function (view) {
              (0, _pipeline.reuse)(view, transition);
            });

            // the root of the chain that needs to be replaced
            // is the top-most non-reusable view.
            if (daq.length) {
              var view = daq[daq.length - 1];
              var depth = reuseQueue ? reuseQueue.length : 0;
              (0, _pipeline.activate)(view, transition, depth, cb);
            } else {
              cb();
            }
          });
        });
      });
    }

    /**
     * Asynchronously and sequentially apply a function to a
     * queue.
     *
     * @param {Array} queue
     * @param {Function} fn
     * @param {Function} cb
     */

  }, {
    key: 'runQueue',
    value: function runQueue(queue, fn, cb) {
      var transition = this;
      step(0);
      function step(index) {
        if (index >= queue.length) {
          cb();
        } else {
          fn(queue[index], transition, function () {
            step(index + 1);
          });
        }
      }
    }

    /**
     * Call a user provided route transition hook and handle
     * the response (e.g. if the user returns a promise).
     *
     * @param {Function} hook
     * @param {*} [context]
     * @param {Function} [cb]
     * @param {Boolean} [expectBoolean]
     * @param {Function} [cleanup]
     */

  }, {
    key: 'callHook',
    value: function callHook(hook, context, cb, expectBoolean, cleanup) {
      var transition = this;
      var nextCalled = false;

      // advance the transition to the next step
      var next = function next(data) {
        if (nextCalled) {
          (0, _util.warn)('transition.next() should be called only once.');
          return;
        }
        nextCalled = true;
        if (!cb || transition.aborted) {
          return;
        }
        cb(data);
      };

      // abort the transition
      var abort = function abort(back) {
        cleanup && cleanup();
        transition.abort(back);
      };

      // handle errors
      var onError = function onError(err) {
        // cleanup indicates an after-activation hook,
        // so instead of aborting we just let the transition
        // finish.
        cleanup ? next() : abort();
        if (err && !transition.router._suppress) {
          (0, _util.warn)('Uncaught error during transition: ');
          throw err instanceof Error ? err : new Error(err);
        }
      };

      // expose a clone of the transition object, so that each
      // hook gets a clean copy and prevent the user from
      // messing with the internals.
      var exposed = {
        to: transition.to,
        from: transition.from,
        abort: abort,
        next: next,
        redirect: function redirect() {
          transition.redirect.apply(transition, arguments);
        }
      };

      // actually call the hook
      var res = undefined;
      try {
        res = hook.call(context, exposed);
      } catch (err) {
        return onError(err);
      }

      // handle boolean/promise return values
      var resIsPromise = (0, _util.isPromise)(res);
      if (expectBoolean) {
        if (typeof res === 'boolean') {
          res ? next() : abort();
        } else if (resIsPromise) {
          res.then(function (ok) {
            ok ? next() : abort();
          }, onError);
        }
      } else if (resIsPromise) {
        res.then(next, onError);
      }
    }
  }]);

  return RouteTransition;
})();

exports['default'] = RouteTransition;
module.exports = exports['default'];